import sys
import copy
import json
import click
import warnings

if __name__ == '__main__':
    from videoall import *
    from __init__ import __version__
else:
    from videoall import *
    from .__init__ import __version__
warnings.filterwarnings('ignore')

'''basic info'''
BASICINFO = '''************************************************************
Function: 视频下载器 V%s
操作帮助:
    输入r: 重新初始化程序(即返回主菜单)
    输入q: 退出程序
视频保存路径:
    当前路径下的%s文件夹内
************************************************************'''

'''视频下载器'''


class videoall():
    def __init__(self, configpath=None, config=None, **kwargs):
        assert configpath or config, 'configpath of config should be given'
        self.config = loadConfig(configpath) if config is None else config
        self.logger_handle = Logger(self.config['logfilepath'])
        self.supported_sources = self.initializeAllSources()

    def get_video(self, video_path=None):
        if video_path is None:
            # 视频链接输入
            video_path = self.dealInput('请输入视频链接: ')
        # 判断视频链接类型是否支持解析下载
        source = self.findsource(video_path)
        if source is None:
            self.logger_handle.warning('暂不支持解析视频链接: %s' % video_path)
            return None
        # 实例化
        client = source(self.config, self.logger_handle)
        # 视频链接解析
        videoinfos = client.parse(video_path)
        # 视频下载
        client.download(videoinfos)

    '''非开发人员外部调用'''

    def run(self, user_input=None):
        print(BASICINFO % (__version__, self.config.get('savedir')))
        while True:
            self.get_video()


    '''判断视频源'''

    def findsource(self, url):
        for key, source in self.supported_sources.items():
            if source.isurlvalid(url): return source
        return None

    '''初始化所有支持的搜索/下载源'''

    def initializeAllSources(self):
        supported_sources = {
            'ted': Ted, 'cntv': CNTV, 'mgtv': MGTV, 'migu': Migu,
            'pipix': Pipix, 'acfun': AcFun, 'zhihu': Zhihu, 'xigua': Xigua,
            'iqiyi': Iqiyi, 'douyin': Douyin, 'haokan': Haokan, 'bilibili': Bilibili,
            'yinyuetai': Yinyuetai, 'pipigaoxiao': Pipigaoxiao, 'weibo': Weibo, 'ku6': Ku6,
            'baidutieba': BaiduTieba, 'kuaishou': Kuaishou, 'sohu': Sohu,
        }
        return supported_sources

    '''处理用户输入'''

    def dealInput(self, tip=''):
        user_input = input(tip)
        if user_input.lower() == 'q':
            self.logger_handle.info('ByeBye')
            sys.exit()
        elif user_input.lower() == 'r':
            self.supported_sources = self.initializeAllSources()
            self.run()
        else:
            return user_input

    '''str'''

    def __str__(self):
        return 'Welcome to use videodl!\nYou can visit https://github.com/CharlesPikachu/videodl for more details.'


'''cmd直接运行'''


@click.command()
@click.option('-i', '--url', default=None, help='想要下载的视频链接, 若不指定, 则进入videodl终端版')
@click.option('-l', '--logfilepath', default='videodl.log', help='日志文件保存的路径')
@click.option('-p', '--proxies', default='{}', help='设置的代理')
@click.option('-s', '--savedir', default='videos', help='视频保存的文件夹')
def videodlcmd(url, logfilepath, proxies, savedir):
    config = {
        'logfilepath': logfilepath,
        'proxies': json.loads(proxies),
        'savedir': savedir,
    }
    dl_client = videoall(config=config)
    if url is None:
        dl_client.run()
    else:
        print(dl_client)
        source = dl_client.findsource(url)
        client = source(dl_client.config, dl_client.logger_handle)
        videoinfos = client.parse(url)
        client.download(videoinfos)


'''run'''
if __name__ == '__main__':
    import os

    rootdir = os.path.split(os.path.abspath(__file__))[0]
    dl_client = videoall(os.path.join(rootdir, 'config.json'))
    dl_client.run()



import re
import json
import time
import random
import requests
from .base import Base
from ..utils import filterBadCharacter


'''抖音视频下载器类'''
class Douyin(Base):
    def __init__(self, config, logger_handle, **kwargs):
        super(Douyin, self).__init__(config, logger_handle, **kwargs)
        self.source = 'douyin'
        self.__initialize()
    '''视频解析'''
    def parse(self, url):
        url = re.findall(r'http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+', url)[0]
        response = self.session.get(url)
        if response.url[:28] == 'https://www.douyin.com/user/':
            videoinfos = self.multiparse(response)
        else:
            videoinfos = self.singleparse(response)
        return videoinfos
    '''批量下载用户的视频'''
    def multiparse(self, response):
        videoinfos = []
        key = re.findall(r'/user/(.*?)\?', str(response.url))[0]
        if not key: key = response.url[28: 83]
        page_count, max_cursor = 35, 0
        while True:
            url = self.uid_url.format('post', key, page_count, max_cursor)
            response = self.session.get(url, headers=self.headers)
            response_json = json.loads(response.content.decode())
            max_cursor, aweme_list = response_json['max_cursor'], response_json['aweme_list']
            if max_cursor == 0: break
            for idx in range(min(page_count, len(aweme_list))):
                download_url = str(aweme_list[idx].get('video', {}).get('play_addr', {}).get('url_list', [''])[0])
                videoinfo = {
                    'source': self.source,
                    'aweme_id': aweme_list[idx].get('aweme_id', None),
                    'download_url': download_url,
                    'savedir': self.config['savedir'],
                    'savename': filterBadCharacter(str(aweme_list[idx].get('desc', f'视频走丢啦_{time.time()}'))),
                    'ext': 'mp4',
                }
                if videoinfo['download_url']: videoinfos.append(videoinfo)
            time.sleep(random.random() + 0.2)
        return videoinfos
    '''下载单个视频'''
    def singleparse(self, response):
        url = self.iteminfo_url.format(re.findall(r'video/(\d+)?', str(response.url))[0])
        response_json = json.loads(self.session.get(url, headers=self.headers).text)
        try: download_url = str(response_json['item_list'][0]['video']['play_addr']['url_list'][0]).replace('playwm','play')
        except: return []
        videoinfo = {
            'source': self.source,
            'download_url': download_url,
            'savedir': self.config['savedir'],
            'savename': filterBadCharacter(str(response_json.get('item_list', [{}])[0].get('desc', f'视频走丢啦_{time.time()}'))),
            'ext': 'mp4',
        }
        return [videoinfo]
    '''初始化'''
    def __initialize(self):
        self.headers = {
            'User-Agent': 'Mozilla/5.0 (Linux; Android 8.0; Pixel 2 Build/OPD3.170816.012) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.88 Mobile Safari/537.36 Edg/87.0.664.66',
        }
        self.iteminfo_url = 'https://www.iesdouyin.com/web/api/v2/aweme/iteminfo/?item_ids={}'
        self.uid_url = 'https://www.iesdouyin.com/web/api/v2/aweme/{}/?sec_uid={}&count={}&max_cursor={}&aid=1128&_signature=PDHVOQAAXMfFyj02QEpGaDwx1S&dytk='
    '''判断视频链接是否属于该类'''
    @staticmethod
    def isurlvalid(url):
        valid_hosts = ['v.douyin.com']
        for host in valid_hosts:
            if host in url: return True
        return False


import re
import time
from .base import Base
from ..utils import filterBadCharacter


'''央视网视频下载器类'''
class CNTV(Base):
    def __init__(self, config, logger_handle, **kwargs):
        super(CNTV, self).__init__(config, logger_handle, **kwargs)
        self.source = 'cntv'
        self.__initialize()
    '''视频解析'''
    def parse(self, url):
        response = self.session.get(url, headers=self.headers)
        pid = re.findall(r'var guid = "(\w+)"', response.text)[0]
        params = {
            'pid': pid,
            'client': 'flash',
            'im': '0',
            'tsp': str(int(time.time())),
            'vn': '2049',
            'vc': '9BFAED20B5E91364B9B1E5A777565554',
            'uid': 'A15780E361D8094E78EAE9AB2BD31534',
            'wlan': '',
        }
        response = self.session.get(self.getHttpVideoInfo_url, params=params, headers=self.headers)
        response_json = response.json()
        download_url, flag = [], False
        for video_type in response_json['video']:
            for support_type in ['chapters4', 'chapters3', 'chapters2', 'chapters', 'lowChapters']:
                if support_type == video_type:
                    for item in response_json['video'][video_type]:
                        download_url.append(item['url'])
                    flag = True
                    break
            if flag: break
        if len(download_url) == 1: download_url = download_url[0]
        videoinfo = {
            'source': self.source,
            'download_url': download_url,
            'savedir': self.config['savedir'],
            'savename': filterBadCharacter(response_json.get('title', f'视频走丢啦_{time.time()}')),
            'ext': 'mp4',
            'split_ext': 'mp4',
        }
        return [videoinfo]
    '''初始化'''
    def __initialize(self):
        self.headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.159 Safari/537.36',
        }
        self.getHttpVideoInfo_url = 'https://vdn.apps.cntv.cn/api/getHttpVideoInfo.do?'
    '''判断视频链接是否属于该类'''
    @staticmethod
    def isurlvalid(url):
        valid_hosts = ['v.cctv.com']
        for host in valid_hosts:
            if host in url: return True
        return False


import re
from .base import Base
from ..utils import filterBadCharacter


'''皮皮虾视频下载器类'''
class Pipix(Base):
    def __init__(self, config, logger_handle, **kwargs):
        super(Pipix, self).__init__(config, logger_handle, **kwargs)
        self.source = 'pipix'
        self.__initialize()
    '''视频解析'''
    def parse(self, url):
        item_id = re.findall(r'item/(\d+)', url)[0]
        url = self.detail_url.format(item_id)
        response = self.session.get(url)
        response_json = response.json()
        videoinfo = {
            'source': self.source,
            'download_url': response_json['data']['item']['video']['video_download']['url_list'][0]['url'],
            'savedir': self.config['savedir'],
            'savename': filterBadCharacter(response_json['data']['item']['share']['title']),
            'ext': 'mp4',
        }
        return [videoinfo]
    '''初始化'''
    def __initialize(self):
        self.headers = {
            'User-Agent': 'Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A5376e Safari/8536.25',
        }
        self.detail_url = 'https://h5.pipix.com/bds/webapi/item/detail/?item_id={}&source=share'
        self.session.headers.update(self.headers)
    '''判断视频链接是否属于该类'''
    @staticmethod
    def isurlvalid(url):
        valid_hosts = ['h5.pipix.com']
        for host in valid_hosts:
            if host in url: return True
        return False


import re
import json
import time
from .base import Base
from ..utils import filterBadCharacter


'''知乎视频下载器类'''
class Zhihu(Base):
    def __init__(self, config, logger_handle, **kwargs):
        super(Zhihu, self).__init__(config, logger_handle, **kwargs)
        self.source = 'zhihu'
        self.__initialize()
    '''视频解析'''
    def parse(self, url):
        response = self.session.get(url, headers=self.headers)
        title = re.findall(r'data-react-helmet="true">(.*?)</title>', response.text)
        if title: title = title[0]
        else: title = f'视频走丢啦_{time.time()}'
        videoinfos = []
        for idx, video_id in enumerate(re.findall(r'<a class="video-box" href="\S+video/(\d+)"', response.text)):
            info = json.loads(self.session.get(self.videoapi_url.format(video_id), headers=self.headers).text)
            play_list = info['playlist']
            info = play_list.get('hd', play_list.get('sd', play_list.get('ld', None)))
            if info is None: continue
            download_url = info['play_url']
            videoinfo = {
                'source': self.source,
                'download_url': download_url,
                'savedir': self.config['savedir'],
                'savename': filterBadCharacter(title + '_' + video_id),
                'ext': 'mp4',
            }
            videoinfos.append(videoinfo)
        return videoinfos
    '''初始化'''
    def __initialize(self):
        self.headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.159 Safari/537.36',
        }
        self.videoapi_url = 'https://lens.zhihu.com/api/videos/{}'
    '''判断视频链接是否属于该类'''
    @staticmethod
    def isurlvalid(url):
        valid_hosts = ['zhihu.com']
        for host in valid_hosts:
            if host in url: return True
        return False


import os
import time
import math
import shutil
import requests
import subprocess
from .misc import touchdir
from alive_progress import alive_bar


'''下载器类'''
class Downloader():
    def __init__(self, videoinfo, session=None, **kwargs):
        self.videoinfo = videoinfo
        self.session = requests.Session() if session is None else session
        self.__setheaders(videoinfo['source'])
    '''外部调用'''
    def start(self):
        if self.videoinfo['ext'] in ['mp4']: 
            if isinstance(self.videoinfo['download_url'], list):
                return self.defaultmultivideopartdownload()
            else:
                return self.defaultsinglevideodownload()
        elif self.videoinfo['ext'] in ['m3u8']: 
            if isinstance(self.videoinfo['download_url'], list):
                return self.m3u8download('requests')
            else:
                return self.m3u8download('ffmpeg')
        else: 
            raise NotImplementedError('Unsupport download file type %s' % self.videoinfo['ext'])
    '''默认的下载函数'''
    def defaultsinglevideodownload(self):
        videoinfo, session, headers = self.videoinfo.copy(), self.session, self.headers.copy()
        touchdir(videoinfo['savedir'])
        with session.get(videoinfo['download_url'], headers=headers, stream=True) as response:
            if response.status_code not in [200]: return False
            total_size, chunk_size, downloaded_size = int(response.headers['content-length']), videoinfo.get('chunk_size', 1024), 0
            savepath = os.path.join(videoinfo['savedir'], f"{videoinfo['savename']}.{videoinfo['ext']}")
            text, fp = '[FileSize]: %0.2fMB/%0.2fMB', open(savepath, 'wb')
            with alive_bar(manual=True) as bar:
                for chunk in response.iter_content(chunk_size=chunk_size):
                    if not chunk: continue
                    fp.write(chunk)
                    downloaded_size += chunk_size
                    bar.text(text % (downloaded_size / 1024 / 1024, total_size / 1024 / 1024))
                    bar(min(downloaded_size / total_size, 1))
        return True
    '''下载多个mp4文件'''
    def defaultmultivideopartdownload(self):
        videoinfo = self.videoinfo.copy()
        touchdir(videoinfo['savedir'])
        savepath = os.path.join(videoinfo['savedir'], videoinfo['savename']+'.mp4')
        savepath = savepath.replace(' ', '')
        tmp_dir = f'tmp_{int(time.time())}'
        os.mkdir(tmp_dir)
        fp = open(f'{tmp_dir}/filelist.txt', 'w')
        for idx, url in enumerate(videoinfo['download_url']):
            self.videoinfo['download_url'] = url
            self.videoinfo['savedir'] = tmp_dir
            self.videoinfo['savename'] = f'{idx}'
            self.videoinfo['ext'] = self.videoinfo['split_ext']
            self.defaultsinglevideodownload()
            fp.write(f'file {idx}.{self.videoinfo["ext"]}\n')
        fp.close()
        self.videoinfo = videoinfo
        p = subprocess.Popen(f'ffmpeg -f concat -safe 0 -i {tmp_dir}/filelist.txt -y {savepath}')
        while True:
            if subprocess.Popen.poll(p) is not None: 
                shutil.rmtree(tmp_dir)
                return True
    '''下载m3u8文件'''
    def m3u8download(self, mode='ffmpeg'):
        assert mode in ['ffmpeg', 'requests']
        videoinfo = self.videoinfo.copy()
        touchdir(videoinfo['savedir'])
        savepath = os.path.join(videoinfo['savedir'], videoinfo['savename']+'.mp4')
        savepath = savepath.replace(' ', '')
        if mode == 'ffmpeg':
            download_url = videoinfo['download_url']
            if 'ffmpeg_headers' in videoinfo:
                ffmpeg_headers = videoinfo['ffmpeg_headers']
                p = subprocess.Popen(f'ffmpeg -headers "{ffmpeg_headers}" -i "{download_url}" {savepath}')
            else:
                p = subprocess.Popen(f'ffmpeg -i "{download_url}" {savepath}')
            while True:
                if subprocess.Popen.poll(p) is not None: 
                    return True
        else:
            tmp_dir = f'tmp_{int(time.time())}'
            os.mkdir(tmp_dir)
            fp = open(f'{tmp_dir}/filelist.txt', 'w')
            for idx, url in enumerate(videoinfo['download_url']):
                self.videoinfo['download_url'] = url
                self.videoinfo['savedir'] = tmp_dir
                self.videoinfo['savename'] = f'{idx}'
                self.videoinfo['ext'] = self.videoinfo['split_ext']
                self.defaultsinglevideodownload()
                fp.write(f'file {idx}.{self.videoinfo["ext"]}\n')
            fp.close()
            self.videoinfo = videoinfo
            p = subprocess.Popen(f'ffmpeg -f concat -safe 0 -i {tmp_dir}/filelist.txt -y {savepath}')
            while True:
                if subprocess.Popen.poll(p) is not None: 
                    shutil.rmtree(tmp_dir)
                    return True
    '''设置请求头'''
    def __setheaders(self, source):
        self.douyin_headers = {
            'User-Agent': 'Mozilla/5.0 (Linux; Android 8.0; Pixel 2 Build/OPD3.170816.012) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.88 Mobile Safari/537.36 Edg/87.0.664.66',
        }
        self.bilibili_headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/94.0.4606.81 Safari/537.36',
            'Referer': 'http://player.bilibili.com/',
        }
        self.yinyuetai_headers = {}
        if hasattr(self, f'{source}_headers'):
            self.headers = getattr(self, f'{source}_headers')
        else:
            self.headers = {
                'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.159 Safari/537.36',
            }


import re
import time
from .base import Base
from ..utils import filterBadCharacter


'''百度贴吧视频下载器类'''
class BaiduTieba(Base):
    def __init__(self, config, logger_handle, **kwargs):
        super(BaiduTieba, self).__init__(config, logger_handle, **kwargs)
        self.source = 'baidutieba'
        self.__initialize()
    '''视频解析'''
    def parse(self, url):
        response = self.session.get(url, headers=self.headers)
        title = re.findall(r"'threadTitle': '(.*?)'", response.text)
        videoinfos, download_urls = [], re.findall(r'data-video="(.*?)"', response.text)
        for idx, download_url in enumerate(download_urls):
            if idx == 0: idx = ''
            videoinfo = {
                'source': self.source,
                'download_url': download_url,
                'savedir': self.config['savedir'],
                'savename': filterBadCharacter(title[0] + str(idx) if title else f'视频走丢啦{idx}_{time.time()}'),
                'ext': 'mp4',
            }
            videoinfos.append(videoinfo)
        return videoinfos
    '''初始化'''
    def __initialize(self):
        self.headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.159 Safari/537.36',
        }
    '''判断视频链接是否属于该类'''
    @staticmethod
    def isurlvalid(url):
        valid_hosts = ['tieba.baidu.com']
        for host in valid_hosts:
            if host in url: return True
        return False


from .utils import (
    touchdir, seconds2hms, loadConfig, filterBadCharacter, Logger, printTable, Downloader
)
from .sources import (
    Ted, CNTV, MGTV, Migu, Pipix, AcFun, Zhihu, Xigua, Iqiyi, Douyin, Haokan, Bilibili, Pipigaoxiao,
    Yinyuetai, Weibo, BaiduTieba, Ku6, Kuaishou, Sohu
)


import time
import uuid
import base64
from .base import Base
from ..utils import filterBadCharacter


'''芒果TV下载器类'''
class MGTV(Base):
    def __init__(self, config, logger_handle, **kwargs):
        super(MGTV, self).__init__(config, logger_handle, **kwargs)
        self.source = 'mgtv'
        self.__initialize()
    '''视频解析'''
    def parse(self, url):
        # 计算tk2
        tk2 = bytes(f'did={self.did}|pno={self.pno}|ver=0.3.0301|clit={int(time.time())}'.encode())
        tk2 = base64.b64encode(tk2).decode().replace('/\+/g', '_').replace('/\//g', '~').replace('/=/g', '-')
        tk2 = list(' '.join(tk2).split())
        tk2.reverse()
        tk2 = ''.join(tk2)
        # 计算pm2
        params = {
            'did': self.did,
            'suuid': uuid.uuid4(),
            'cxid': '',
            'tk2': tk2,
            'type': 'pch5',
            'video_id': url.split('/', 5)[-1].split('.')[0],
            '_support': '10000000',
            'auth_mode': '',
            'src': '',
            'abroad': '',
        }
        response = self.session.get(self.video_url, params=params)
        response_json = response.json()
        title = response_json['data'].get('info', {}).get('title', f'视频走丢啦_{time.time()}')
        pm2 = response_json['data']['atc']['pm2']
        # 进行请求, 默认下载清晰度最低的, 因为高清和蓝光的不登陆的话无权限下载
        params = {
            '_support': '10000000',
            'tk2': tk2,
            'pm2': pm2,
            'video_id': url.split('/', 5)[-1].split('.')[0],
            'type': 'pch5',
            'auth_mode': '',
            'src': '',
            'abroad': '',
        }
        response = self.session.get(self.getSource_url, headers=self.headers, params=params)
        response_json = response.json()
        flag = False
        for domian in response_json['data']['stream_domain']:
            for item in response_json['data']['stream']:
                if item['url']:
                    stream_url = domian + item['url']
                    response = self.session.get(stream_url)
                    if response.json().get('info', ''): 
                        download_url = response.json()['info']
                        flag = True
                        break
            if flag: break
        videoinfo = {
            'source': self.source,
            'download_url': download_url,
            'savedir': self.config['savedir'],
            'savename': filterBadCharacter(title),
            'ext': 'm3u8',
            'ffmpeg_headers': f'Referer: {url}'
        }
        return [videoinfo]
    '''初始化'''
    def __initialize(self):
        self.headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.159 Safari/537.36',
        }
        self.video_url = 'https://pcweb.api.mgtv.com/player/video'
        self.getSource_url = 'https://pcweb.api.mgtv.com/player/getSource'
        self.pno = '1030'
        self.did = 'e6e13014-393b-43e7-b6be-2323e4960939'
    '''判断视频链接是否属于该类'''
    @staticmethod
    def isurlvalid(url):
        valid_hosts = ['mgtv.com']
        for host in valid_hosts:
            if host in url: return True
        return False


import re
import json
import time
from .base import Base
from ..utils import filterBadCharacter


'''好看视频下载器类'''
class Haokan(Base):
    def __init__(self, config, logger_handle, **kwargs):
        super(Haokan, self).__init__(config, logger_handle, **kwargs)
        self.source = 'haokan'
        self.__initialize()
    '''视频解析'''
    def parse(self, url):
        response = self.session.get(url, headers=self.headers)
        response_json = json.loads(re.findall(r'PRELOADED_STATE__\s=\s(.*?);', response.text)[0])
        download_url = response_json['curVideoMeta']['playurl']
        videoinfo = {
            'source': self.source,
            'download_url': download_url,
            'savedir': self.config['savedir'],
            'savename': filterBadCharacter(response_json.get('curVideoMeta', {}).get('title', f'视频走丢啦_{time.time()}')),
            'ext': 'mp4',
        }
        return [videoinfo]
    '''初始化'''
    def __initialize(self):
        self.headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.159 Safari/537.36',
        }
    '''判断视频链接是否属于该类'''
    @staticmethod
    def isurlvalid(url):
        valid_hosts = ['haokan.baidu.com']
        for host in valid_hosts:
            if host in url: return True
        return False


import re
import time
from .base import Base
from ..utils import filterBadCharacter


'''B站视频下载器类'''
class Bilibili(Base):
    def __init__(self, config, logger_handle, **kwargs):
        super(Bilibili, self).__init__(config, logger_handle, **kwargs)
        self.source = 'bilibili'
        self.__initialize()
    '''视频解析'''
    def parse(self, url):
        response = self.session.get(url, headers=self.headers)
        bv = re.compile('BV..........').search(url).group()
        response = self.session.get(self.pagelist_url.format(bv), headers=self.headers)
        response_json = response.json()
        cid_list = [item['cid'] for item in response_json['data']]
        titles = [item.get('part', f'视频走丢啦_{time.time()}') for item in response_json['data']]
        download_urls = []
        for cid in cid_list:
            response = self.session.get(self.play_url.format(cid, bv), headers=self.headers)
            response_json = response.json()
            for item in response_json['data']['durl']:
                download_urls.append(item['url'])
        assert len(titles) == len(download_urls)
        videoinfos = []
        for idx, download_url in enumerate(download_urls):
            videoinfo = {
                'source': self.source,
                'download_url': download_url,
                'savedir': self.config['savedir'],
                'savename': filterBadCharacter(titles[idx]),
                'ext': 'mp4',
            }
            videoinfos.append(videoinfo)
        return videoinfos
    '''初始化'''
    def __initialize(self):
        self.headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.159 Safari/537.36',
        }
        self.pagelist_url = 'https://api.bilibili.com/x/player/pagelist?bvid={}&jsonp=jsonp'
        self.play_url = 'http://api.bilibili.com/x/player/playurl?&cid={}&bvid={}&qn=80&fnval=0&fnver=0&fourk=1'
    '''判断视频链接是否属于该类'''
    @staticmethod
    def isurlvalid(url):
        valid_hosts = ['bilibili.com/video']
        for host in valid_hosts:
            if host in url: return True
        return False


from .ku6 import Ku6
from .ted import Ted
from .sohu import Sohu
from .cntv import CNTV
from .mgtv import MGTV
from .migu import Migu
from .pipix import Pipix
from .acfun import AcFun
from .zhihu import Zhihu
from .xigua import Xigua
from .iqiyi import Iqiyi
from .weibo import Weibo
from .douyin import Douyin
from .haokan import Haokan
from .kuaishou import Kuaishou
from .bilibili import Bilibili
from .yinyuetai import Yinyuetai
from .baidutieba import BaiduTieba
from .pipigaoxiao import Pipigaoxiao


import re
import json
from .base import Base
from ..utils import filterBadCharacter


'''皮皮搞笑视频下载器类'''
class Pipigaoxiao(Base):
    def __init__(self, config, logger_handle, **kwargs):
        super(Pipigaoxiao, self).__init__(config, logger_handle, **kwargs)
        self.source = 'pipigaoxiao'
        self.__initialize()
    '''视频解析'''
    def parse(self, url):
        pattern = re.compile(r'http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+', re.S)
        url = re.findall(pattern, url)[0]
        self.headers['Referer'] = url
        try:
            mid = re.findall('mid=(\d+)', url, re.S)[0]
            pid = re.findall('pid=(\d+)', url, re.S)[0]
        except:
            mid = ''
            pid = url.split('/')[-1]
        data = {
            'mid': int(mid) if mid else 'null',
            'pid': int(pid),
            'type': 'post',
        }
        response = self.session.post(self.content_url, data=json.dumps(data), headers=self.headers)
        response_json = response.json()
        download_url = response_json['data']['post']['videos'][str(response_json['data']['post']['imgs'][0]['id'])]['url']
        title = response_json['data']['post']['content'].replace('\n', '')
        videoinfo = {
            'source': self.source,
            'download_url': download_url,
            'savedir': self.config['savedir'],
            'savename': filterBadCharacter(title),
            'ext': 'mp4',
        }
        return [videoinfo]
    '''初始化'''
    def __initialize(self):
        self.headers = {
            'Host': 'share.ippzone.com',
            'Origin': 'http://share.ippzone.com',
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/98.0.4758.82 Safari/537.36',
        }
        self.content_url = 'https://h5.ippzone.com/ppapi/share/fetch_content'
    '''判断视频链接是否属于该类'''
    @staticmethod
    def isurlvalid(url):
        valid_hosts = ['h5.ippzone.com', 'share.ippzone.com']
        for host in valid_hosts:
            if host in url: return True
        return False


import time
from .base import Base
from ..utils import filterBadCharacter


'''咪咕视频下载器类'''
class Migu(Base):
    def __init__(self, config, logger_handle, **kwargs):
        super(Migu, self).__init__(config, logger_handle, **kwargs)
        self.source = 'migu'
        self.__initialize()
    '''视频解析'''
    def parse(self, url):
        # 获得下载链接
        params = {
            'contId': url.split('?')[-1].split('=')[-1],
            'clientId': '2e2919355618c3465540e0d7b0e98642',
        }
        response = self.session.get(self.play_url, params=params, headers=self.headers)
        response_json = response.json()
        download_url, params_dict = response_json['body']['urlInfo']['url'], {}
        # 获得真实下载链接
        for c in download_url.split('&'):
            params_dict[c.split('=')[0]] = c.split('=')[1]
        data, p, ddCalcu = list(params_dict['puData']), 0, []
        while (2 * p) < len(data):
            ddCalcu.append(data[len(data) - p - 1])
            if p < len(data) - p - 1: ddCalcu.append(params_dict['puData'][p])
            if p == 1: ddCalcu.append('e')
            if p == 2: ddCalcu.append(list(params_dict['timestamp'])[6])
            if p == 3: ddCalcu.append(list(params_dict['ProgramID'])[2])
            if p == 4: ddCalcu.append(list(params_dict['Channel_ID'])[len(list(params_dict['Channel_ID'])) - 4])
            p += 1
        ddCalcu = ''.join(ddCalcu)
        # 构造videoinfo
        download_url = f'{download_url}&ddCalcu={ddCalcu}'
        title = response_json.get('body', {}).get('content', {}).get('contName', f'视频走丢啦_{time.time()}')
        videoinfo = {
            'source': self.source,
            'download_url': download_url,
            'savedir': self.config['savedir'],
            'savename': filterBadCharacter(title),
            'ext': 'mp4',
        }
        return [videoinfo]
    '''初始化'''
    def __initialize(self):
        self.headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.159 Safari/537.36',
        }
        self.play_url = 'https://webapi.miguvideo.com/gateway/playurl/v3/play/playurl'
    '''判断视频链接是否属于该类'''
    @staticmethod
    def isurlvalid(url):
        valid_hosts = ['miguvideo.com']
        for host in valid_hosts:
            if host in url: return True
        return False


import setuptools

with open('README.md', 'r', encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name='videoall',
    version='0.0.5',
    author='C灵C',
    author_email='c0c@cocpy.com',
    description='All video download',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/cocpy/VideoAll',
    packages=setuptools.find_packages(),
    install_requires=[

    ],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
)


import re
import time
from .base import Base
from ..utils import filterBadCharacter


'''搜狐视频下载器类'''
class Sohu(Base):
    def __init__(self, config, logger_handle, **kwargs):
        super(Sohu, self).__init__(config, logger_handle, **kwargs)
        self.source = 'sohu'
        self.__initialize()
    '''视频解析'''
    def parse(self, url):
        if re.match(r'http://share.vrs.sohu.com', url):
            vid = r1('id=(\d+)', url)
        else:
            response = self.session.get(url, headers=self.headers)
            vid = re.findall(r'\Wvid\s*[\:=]\s*[\'"]?(\d+)[\'"]?', response.text) or re.findall(r'bid:\'(\d+)\',', response.text) or re.findall(r'bid=(\d+)', response.text)
        if not vid: return []
        params = {'vid': vid[0]}
        response = self.session.get(self.flash_url, params=params, headers=self.headers)
        response_json = response.json()
        if response_json and response_json.get('data', ''):
            for qtyp in ['oriVid', 'superVid', 'highVid', 'norVid', 'relativeId']:
                hqvid = response_json['data'][qtyp]
                if hqvid != 0 and hqvid != vid[0]:
                    params = {'vid': hqvid}
                    response = self.session.get(self.flash_url, params=params, headers=self.headers)
                    if 'allot' not in response.json(): continue
                    response_json = response.json()
                    break
        else:
            url = f'http://my.tv.sohu.com/play/videonew.do?vid={vid[0]}&referer=http://my.tv.sohu.com'
            response = self.session.get(url, headers=self.headers)
            response_json = response.json()
        title = response_json['data']['tvName']
        assert len(response_json['data']['clipsURL']) == len(response_json['data']['clipsBytes']) == len(response_json['data']['su'])
        download_url = []
        for su, ck in zip(response_json['data']['su'], response_json['data']['ck']):
            download_url.append(self.converttorealurl(su, ck, response_json['data']['ch']))
        if len(download_url) == 1: download_url = download_url[0]
        videoinfo = {
            'source': self.source,
            'download_url': download_url,
            'savedir': self.config['savedir'],
            'savename': filterBadCharacter(title or f'视频走丢啦_{time.time()}'),
            'ext': 'mp4',
            'split_ext': 'mp4',
        }
        return [videoinfo]
    '''获得真实下载链接'''
    def converttorealurl(self, su, ck, ch):
        url = f'https://data.vod.itc.cn/ip?new={su}&num=1&key={ck}&ch={ch}&pt=1&pg=2&prod=h5n'
        response = self.session.get(url, headers=self.headers)
        return response.json()['servers'][0]['url']
    '''初始化'''
    def __initialize(self):
        self.headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.159 Safari/537.36',
        }
        self.flash_url = 'https://hot.vrs.sohu.com/vrs_flash.action'
    '''判断视频链接是否属于该类'''
    @staticmethod
    def isurlvalid(url):
        valid_hosts = ['sohu.com']
        for host in valid_hosts:
            if host in url: return True
        return False


import requests
from ..utils import Downloader


'''视频下载器基类'''
class Base():
    def __init__(self, config, logger_handle, **kwargs):
        self.source = None
        self.session = requests.Session()
        self.session.proxies.update(config['proxies'])
        self.config = config
        self.logger_handle = logger_handle
    '''视频解析'''
    def parse(self, url):
        raise NotImplementedError('not be implemented')
    '''歌曲下载'''
    def download(self, videoinfos):
        for videoinfo in videoinfos:
            self.logger_handle.info('正在从%s下载 ——> %s' % (self.source, videoinfo['savename']))
            task = Downloader(videoinfo.copy(), self.session)
            if task.start():
                self.logger_handle.info('成功从%s下载到了 ——> %s' % (self.source, videoinfo['savename']))
            else:
                self.logger_handle.info('无法从%s下载 ——> %s' % (self.source, videoinfo['savename']))
    '''初始化'''
    def __initialize(self):
        raise NotImplementedError('not be implemented')
    '''返回类信息'''
    def __repr__(self):
        return 'Video Source: %s' % self.source
    '''判断视频链接是否属于该类'''
    @staticmethod
    def isurlvalid(url):
        raise NotImplementedError('not be implemented')


import re
import time
from .base import Base
from ..utils import filterBadCharacter


'''酷6视频下载器类'''
class Ku6(Base):
    def __init__(self, config, logger_handle, **kwargs):
        super(Ku6, self).__init__(config, logger_handle, **kwargs)
        self.source = 'ku6'
        self.__initialize()
    '''视频解析'''
    def parse(self, url):
        response = self.session.get(url, headers=self.headers)
        download_url = re.findall(r'src: "(https://.*?)"', response.text)
        if not download_url: return []
        title = re.findall(r'document.title = "(.*?)";', response.text)
        videoinfo = {
            'source': self.source,
            'download_url': download_url[0],
            'savedir': self.config['savedir'],
            'savename': filterBadCharacter(title[0] if title else f'视频走丢啦_{time.time()}'),
            'ext': 'mp4',
        }
        return [videoinfo]
    '''初始化'''
    def __initialize(self):
        self.headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.159 Safari/537.36',
        }
    '''判断视频链接是否属于该类'''
    @staticmethod
    def isurlvalid(url):
        valid_hosts = ['ku6.com']
        for host in valid_hosts:
            if host in url: return True
        return False